require "import"
import "android.widget.*"
import "android.view.*"
import "android.graphics.PixelFormat"
import "android.content.Context"
import "android.media.MediaRecorder"
import "android.media.MediaPlayer"
import "java.io.File"
import "java.text.SimpleDateFormat"
import "java.util.Date"

local isRecording = false
local mediaRecorder = nil
local mediaPlayer = nil
local currentRecordingPath = nil

-- Create folder for recordings
local folderName = "Magic Voice Recorder"
local folderPath = "/storage/emulated/0/" .. folderName
local folder = File(folderPath)
if not folder.exists() then
    folder.mkdirs()
end

-- Function to start recording
function startRecording()
    mediaRecorder = MediaRecorder()
    mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC)
    mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4)
    mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC)
    mediaRecorder.setAudioEncodingBitRate(128000)
    mediaRecorder.setAudioSamplingRate(44100)

    local dateFormat = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault())
    local currentTime = dateFormat.format(Date())
    currentRecordingPath = folderPath .. "/recording_" .. currentTime .. ".mp3"
    mediaRecorder.setOutputFile(currentRecordingPath)

    mediaRecorder.prepare()
    mediaRecorder.start()
    isRecording = true
    recordButton.text = "Stop Recording"
    updateStorageLocationText()
end

-- Function to stop recording
function stopRecording()
    if mediaRecorder ~= nil then
        mediaRecorder.stop()
        mediaRecorder.release()
        mediaRecorder = nil
    end
    isRecording = false
    recordButton.text = "Start Recording"
    updateStorageLocationText()
end

-- Function to play recording
function playRecording()
    if currentRecordingPath ~= nil then
        mediaPlayer = MediaPlayer()
        mediaPlayer.setDataSource(currentRecordingPath)
        mediaPlayer.prepare()
        mediaPlayer.start()
        
        mediaPlayer.setOnCompletionListener(function()
            playButton.text = "Play"
        end)
        
        playButton.text = "Playing..."
    end
end

-- Function to delete recording
function deleteRecording()
    if currentRecordingPath ~= nil then
        local file = File(currentRecordingPath)
        if file.exists() then
            file.delete()
            currentRecordingPath = nil
            Toast.makeText(service, "Recording deleted", Toast.LENGTH_SHORT).show()
            updateStorageLocationText()
        end
    end
end

-- Function to update storage location text
function updateStorageLocationText()
    if currentRecordingPath then
        storageLocationText.text = "Current recording: " .. currentRecordingPath
    else
        storageLocationText.text = "Recordings are stored in: " .. folderPath
    end
end

-- Create main layout
local layout = {
    LinearLayout,
    orientation = "vertical",
    layout_width = "match_parent",
    layout_height = "match_parent",
    padding = "16dp",
    backgroundColor = "#FFFFFF",
    {
        TextView,
        id = "storageLocationText",
        layout_width = "match_parent",
        layout_height = "wrap_content",
        textSize = "12sp",
        padding = "8dp",
    },
    {
        Button,
        id = "recordButton",
        text = "Start Recording",
        layout_width = "match_parent",
        layout_height = "wrap_content",
    },
    {
        Button,
        id = "playButton",
        text = "Play",
        layout_width = "match_parent",
        layout_height = "wrap_content",
    },
    {
        Button,
        id = "deleteButton",
        text = "Delete",
        layout_width = "match_parent",
        layout_height = "wrap_content",
    },
    {
        Button,
        id = "closeButton",
        text = "Close",
        layout_width = "match_parent",
        layout_height = "wrap_content",
    }
}

-- Create and show floating window
local windowManager = service.getSystemService(Context.WINDOW_SERVICE)
local params = WindowManager.LayoutParams()
params.type = WindowManager.LayoutParams.TYPE_ACCESSIBILITY_OVERLAY
params.format = PixelFormat.TRANSLUCENT
params.flags = WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE
params.width = WindowManager.LayoutParams.WRAP_CONTENT
params.height = WindowManager.LayoutParams.WRAP_CONTENT
params.gravity = Gravity.TOP | Gravity.LEFT

local view = loadlayout(layout)
windowManager.addView(view, params)

-- Set up record button
recordButton.onClick = function()
    if isRecording then
        stopRecording()
    else
        startRecording()
    end
end

-- Set up play button
playButton.onClick = function()
    if mediaPlayer ~= nil and mediaPlayer.isPlaying() then
        mediaPlayer.stop()
        mediaPlayer.release()
        mediaPlayer = nil
        playButton.text = "Play"
    else
        playRecording()
    end
end

-- Set up delete button
deleteButton.onClick = function()
    deleteRecording()
end

-- Set up close button
closeButton.onClick = function()
    if mediaRecorder ~= nil then
        stopRecording()
    end
    if mediaPlayer ~= nil then
        mediaPlayer.release()
    end
    windowManager.removeView(view)
end

-- Initialize storage location text
updateStorageLocationText()